
# q-expansion of newform 25.8.a.b, downloaded from the LMFDB on 13 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 25
weight = 8
poly_data = [-19, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[0, 2], 1]]

hecke_ring_character_values = None
aps_data = [[-10, 1], [-10, -8], [0, 0], [50, 56], [2272, -400], [-1770, -608], [13670, -1184], [19380, 320], [62070, -408], [-36130, -19520], [153412, 2800], [61510, 25536], [132182, 56800], [-211650, 43192], [52730, 45496], [1195790, -53408], [-560060, 227360], [1128522, -160000], [-2258230, -79384], [310892, 70000], [-2284530, -226208], [2166520, 472480], [4896510, 490392], [3012810, -317760], [-2304770, 561696], [385902, 379200], [-6314210, -271608], [-5695950, -1091544], [-5215510, 413760], [2534630, 2723392], [22384290, -366824], [21882912, 1011600], [5141310, 3044736], [30411860, -1107360], [-15076750, 5108800], [20394252, 2937200], [-4140050, -1738944], [-23401970, -18408], [18266370, -4605384], [47521470, 5587392], [106656220, -2521120], [21897662, -13584000], [-92322868, -6210800], [110621350, -2901408], [-103208970, -4771104], [-85845000, 11302400], [38220272, -15250800], [-19229930, 28422792], [150104290, 17261576], [-164885930, 6830080], [-241184490, -9866208], [-8286440, -5620960], [-11107218, 33884000], [-78649248, 5934000], [2617350, 37409856], [-104789570, -9268408], [48631930, 25493120], [369610092, -15929200], [229495790, -42620224], [-375787338, 29872800], [-136349290, -9375208], [-144951450, -30676608], [-358084950, 22953256], [-39856428, -14833200], [399005830, 14121792], [-16415330, -52405984], [40245112, -14663600], [-267265890, 65542336], [422963930, -84690104], [-680449650, 18012800], [-173798610, 67420992], [35480040, -32605440], [-792129430, -76477384], [-1172085330, 62281792], [-10915380, 11055680], [716903910, 4523592], [1688366610, -29823360], [642334230, -54507104], [1966830402, 40569600], [-405724210, -48063840], [382321580, 50660320], [-1572761358, 156084800], [273858012, 53931600], [1351994910, 66959392], [-1352981640, -213760], [-1634706450, 65940792], [729958150, -279047200], [1583493750, 250157056], [786404022, 321691200], [56692430, 21674792], [2012583370, -184404184], [702170520, 399317280], [-2558865590, -79380264], [845774432, -483995600], [1596453300, 246972800], [1936269790, 132686792], [122556190, 142933760], [-2558843158, -219699200], [-1644345930, 535029592], [-236657018, 398737600], [-4224918270, -3728504], [1565392950, 469486656], [-124310970, -487600008], [1099652430, 65682720], [-1270515208, 219128400], [-4877164010, 706176], [728657010, 183005736], [-1416849650, -327371008], [5502017600, -229263200], [4021505702, -118476000], [4156802250, 362729656], [3372580230, 568424992], [5617672470, -313509984], [-575819620, -194580480], [1846974012, -215198800], [-2106835218, -116944800], [-5892033850, 391249592], [327765530, -393283304], [-3718767410, 315888992], [306092140, -918406240], [5817663522, 831324800], [-10104690130, -352434208], [1878414390, 149474976], [-3668776290, 459051192], [-411967368, -463478800], [-9525667798, 873140800], [7396260390, -689304640], [1714880280, 884937120], [10712518690, -885390024], [-2080182290, -1042795008], [-1691841540, -604814560], [1480932950, -315738008], [-6568173948, -1205453200], [-6496030050, -837052544], [6644776122, 1203475200], [-4755070, -505922880], [8016221670, -852155808], [23520003810, -58852664], [-19357501970, -641804704], [-3987079510, -1031075840], [-6169525328, -1642523600], [2553211842, -1207732800], [12826122070, -2118739608], [3784942490, -1844448824], [-3333051830, -1600325120], [-339700240, -520326560], [-4253222410, 2409716192], [1897469150, 1447877056], [-2926762860, 3135832160], [17154536230, 371289992], [27855983390, 1111035776], [-10425017138, 2410032800], [-6670356890, -3193763208], [-27072778590, 1170513336], [726243050, 1232547656], [23919731572, -882000400], [-21453756720, -2543320480], [192331670, 1902746080], [-4847276890, -1680031264], [-16149044218, -3785734400], [-27439755270, -311690904], [11529494390, 821550592], [8847546970, -141580584], [-30115391208, -2088692400], [-10692373810, 1063050976], [-8174085090, -4633442808], [-15594654268, -153646800], [1823927630, -2926062304]]
